/*******************************************************************************
 * Copyright (c) 2008 Hallvard Traetteberg.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Hallvard Traetteberg - initial API and implementation
 ******************************************************************************/
package org.eclipse.e4.tm.builder;

import java.net.URL;

import org.eclipse.e4.tm.layouts.LayoutsPackage;
import org.eclipse.e4.tm.styles.StylesPackage;
import org.eclipse.e4.tm.widgets.WidgetsPackage;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.emf.ecore.xmi.impl.XMLResourceFactoryImpl;

public abstract class TmResourceTestCase extends TmTestCase {

	private Resource res;

	protected void setUp(Resource res) throws Exception {
		super.setUp();

		this.res = res;
		builder.build(res, getTopLevel());
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		res.unload();
		res = null;
	}

	protected void setUp(URI uri) throws Exception {

		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("xml", new XMLResourceFactoryImpl());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("xmi", new XMIResourceFactoryImpl());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("ecore", new EcoreResourceFactoryImpl());

		EPackage.Registry.INSTANCE.put(WidgetsPackage.eNS_URI, WidgetsPackage.eINSTANCE);
		EPackage.Registry.INSTANCE.put(StylesPackage.eNS_URI, StylesPackage.eINSTANCE);
		EPackage.Registry.INSTANCE.put(LayoutsPackage.eNS_URI, LayoutsPackage.eINSTANCE);

		EPackage.Registry.INSTANCE.put(org.eclipse.e4.tm.swt.styles.StylesPackage.eNS_URI, org.eclipse.e4.tm.swt.styles.StylesPackage.eINSTANCE);
		EPackage.Registry.INSTANCE.put(org.eclipse.e4.tm.swt.layouts.LayoutsPackage.eNS_URI, org.eclipse.e4.tm.swt.layouts.LayoutsPackage.eINSTANCE);

		ResourceSet resSet = new ResourceSetImpl();
		Resource res = resSet.getResource(uri, true);
		setUp(res);
	}

	protected void setUp() throws Exception {
		String testResourceName = "/" + getClass().getName().replace(".", "/") + ".xmi";
		URL url = getClass().getResource(testResourceName);
		setUp(URI.createURI(url.toString()));
	}

	protected EObject getRoot(EClass eClass) {
		EObject eObject = res.getContents().get(0);
		return (eClass.isInstance(eObject) ? eObject : null);
	}
}
