/*******************************************************************************
 * Copyright (c) 2008 Hallvard Traetteberg.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Hallvard Traetteberg - initial API and implementation
 ******************************************************************************/
package org.eclipse.e4.tm.builder;

import junit.framework.Assert;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import org.eclipse.e4.tm.builder.swt.SwtBuilder;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

public abstract class TmTestCase extends TestCase {

	private Display display;
	private Shell shell;

	protected AbstractBuilder builder;

	protected void setUp() throws Exception {
		super.setUp();
		display = Display.getDefault();
		shell = new org.eclipse.swt.widgets.Shell(display);

		builder = createBuilder();

		URI binUri = URI.createURI(String.valueOf(this.getClass().getResource(".")));
		String binUriString = binUri.toString();
		String pluginId = "org.eclipse.e4.tm.builder.tests";
		int pos = binUriString.indexOf(pluginId);
		if (pos >= 0) {
			String platformPrefix = binUriString.substring(0, pos) + pluginId + "/";
			builder.getUriConverter().getURIMap().put(URI.createURI("platform:/plugin/" + pluginId + "/"), URI.createURI(platformPrefix));
		}
//		System.out.println(URI.createURI("platform:/plugin/" + pluginId + "/") + " -> " + URI.createURI(platformPrefix));
		shell.setSize(600, 600);
		//		shell.open();
		//		while (!shell.isDisposed()) {
		//			if (!display.readAndDispatch())
		//				display.sleep();
		//		}
	}

	protected AbstractBuilder createBuilder() {
		return new SwtBuilder();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		shell.dispose();
		builder.dispose();
	}

	//

	protected org.eclipse.swt.widgets.Composite getTopLevel() {
		return shell;
	}

	protected EObject getChild(EObject comp, int i, EClass c) {
		EList<EObject> contents = comp.eContents();
		junit.framework.Assert.assertTrue(contents.size() > i);
		EObject child = contents.get(i);
		Assert.assertEquals(c, child.eClass());
		return child;
	}

	protected <T> T getChild(org.eclipse.swt.widgets.Composite comp, int i, Class<T> c, boolean assertClass) {
		org.eclipse.swt.widgets.Control[] children = comp.getChildren();
		junit.framework.Assert.assertTrue(children.length > i);
		org.eclipse.swt.widgets.Control control = children[i];
		if (! c.isInstance(control)) {
			Assert.fail();
		}
		if (assertClass) {
			Assert.assertEquals(c, control.getClass());
		}
		return (T)control;
	}
	protected <T> T getChild(org.eclipse.swt.widgets.Composite comp, int i, Class<T> c) {
		return getChild(comp, i, c, true);
	}
	protected <T> T getChild(int i, Class<T> c, boolean assertClass) {
		return getChild(getTopLevel(), i, c, assertClass);
	}
	protected <T> T getChild(int i, Class<T> c) {
		return getChild(i, c, true);
	}

	protected java.util.List getListFeature(EObject eObject, String featureName) {
		Object value = eObject.eGet(eObject.eClass().getEStructuralFeature(featureName));
		Assert.assertTrue(value instanceof java.util.List);
		return (java.util.List)value;
	}

	protected Object setFeature(EObject eObject, String featureName, Object value) {
		eObject.eSet(eObject.eClass().getEStructuralFeature(featureName), value);
		return value;
	}

	protected Object getFeature(EObject eObject, String featureName) {
		return eObject.eGet(eObject.eClass().getEStructuralFeature(featureName));
	}

	//

	public static void doRun(Class c) {
		TestRunner runner = new TestRunner();
		runner.doRun(new TestSuite(c));
	}

	// utility test methods

	protected void testColor(Color col, int expectedRed, int expectedGreen, int expectedBlue) {
		Assert.assertEquals(expectedRed, col.getRed());
		Assert.assertEquals(expectedGreen, col.getGreen());
		Assert.assertEquals(expectedBlue, col.getBlue());
	}
	protected void testColor(Color col1, Color expected) {
		testColor(col1, expected.getRed(), expected.getGreen(), expected.getBlue());
	}
	protected void testColor(Color col1, RGB expected) {
		testColor(col1, expected.red, expected.green, expected.blue);
	}

	protected void testStyle(org.eclipse.swt.widgets.Control control, int style) {
		Assert.assertTrue((control.getStyle() & style) == style);
	}
	
	protected void testFont(Font font, String name, int style, int size) {
		FontData fontData = font.getFontData()[0];
		assertEquals(name, fontData.getName());
		assertEquals(style, fontData.getStyle());
		assertEquals(size, fontData.getHeight());
	}
}
