/*******************************************************************************
 * Copyright (c) 2008 Hallvard Traetteberg.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Hallvard Traetteberg - initial API and implementation
 ******************************************************************************/
package org.eclipse.e4.tm.builder.styles;

import junit.framework.Assert;

import org.eclipse.e4.tm.builder.TmModelTest;
import org.eclipse.e4.tm.builder.TmTestCase;
import org.eclipse.e4.tm.styles.ColorItem;
import org.eclipse.e4.tm.styles.Style;
import org.eclipse.e4.tm.styles.StyleSelector;
import org.eclipse.e4.tm.styles.StylesFactory;
import org.eclipse.e4.tm.styles.StylesPackage;
import org.eclipse.e4.tm.util.UtilPackage;
import org.eclipse.e4.tm.widgets.WidgetsFactory;
import org.eclipse.e4.tm.widgets.WidgetsPackage;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Control;

public class LabelColorTest extends TmModelTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	public void testLabel() {
		WidgetsPackage widgetsPackage = WidgetsPackage.eINSTANCE;
		StylesPackage stylesPackage = StylesPackage.eINSTANCE;
		Style eStyle = StylesFactory.eINSTANCE.createStyle();
		eStyle.setName("label");
		
		ColorItem eColorConstant1 = StylesFactory.eINSTANCE.createColorItem();
		eColorConstant1.setName("foreground");
		eColorConstant1.setSource("white");
		ColorItem eColorConstant2 = StylesFactory.eINSTANCE.createColorItem();
		eColorConstant2.setName("background");
		eColorConstant2.setSource("#FF0000"); // red
		
		eStyle.getStyleItems().add(eColorConstant1);
		eStyle.getStyleItems().add(eColorConstant2);

		StyleSelector eStyleSelector1 = StylesFactory.eINSTANCE.createStyleSelector();
		eStyleSelector1.setProperty("foreground");
		eStyleSelector1.setSelector("foreground");
		StyleSelector eStyleSelector2 = StylesFactory.eINSTANCE.createStyleSelector();
		eStyleSelector2.setProperty("background");
		eStyleSelector2.setSelector("background");

		eStyle.getStyleSelectors().add(eStyleSelector1);
		eStyle.getStyleSelectors().add(eStyleSelector2);

		EObject eLabel = WidgetsFactory.eINSTANCE.create(widgetsPackage.getLabel());
		eLabel.eSet(stylesPackage.getStyled_Style(), eStyle);
		eLabel.eSet(UtilPackage.eINSTANCE.getLabeled_Text(), "label");

		builder.build(eLabel, getTopLevel());

		Control label = getChild(0, Control.class, false);
		Assert.assertNotNull(label);
		testColor(label.getForeground(), label.getDisplay().getSystemColor(SWT.COLOR_WHITE));
		testColor(label.getBackground(), label.getDisplay().getSystemColor(SWT.COLOR_RED));
	}

	public static void main(String[] args) {
		TmTestCase.doRun(LabelColorTest.class);
	}
}
