/*******************************************************************************
 * Copyright (c) 2008 Hallvard Traetteberg.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Hallvard Traetteberg - initial API and implementation
 ******************************************************************************/
package org.eclipse.e4.tm.builder.styles;

import junit.framework.Assert;

import org.eclipse.e4.tm.builder.TmModelTest;
import org.eclipse.e4.tm.builder.TmTestCase;
import org.eclipse.e4.tm.styles.FontItem;
import org.eclipse.e4.tm.styles.Style;
import org.eclipse.e4.tm.styles.StyleSelector;
import org.eclipse.e4.tm.styles.StylesFactory;
import org.eclipse.e4.tm.styles.StylesPackage;
import org.eclipse.e4.tm.widgets.Composite;
import org.eclipse.e4.tm.widgets.Label;
import org.eclipse.e4.tm.widgets.WidgetsFactory;
import org.eclipse.e4.tm.widgets.WidgetsPackage;
import org.eclipse.swt.SWT;

public class LabelFontTest extends TmModelTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	public void testLabel() {
		WidgetsPackage widgetsPackage = WidgetsPackage.eINSTANCE;
		StylesPackage stylesPackage = StylesPackage.eINSTANCE;

		Style eCompStyle = StylesFactory.eINSTANCE.createStyle();
		eCompStyle.setName("label-styles");
		
		FontItem eFontConstant1 = StylesFactory.eINSTANCE.createFontItem();
		eFontConstant1.setName("normal-font");
		eFontConstant1.setSource("normal Times 12");
		FontItem eFontConstant2 = StylesFactory.eINSTANCE.createFontItem();
		eFontConstant2.setName("italic-bold-arial-large-font");
		eFontConstant2.setSource("italic bold Arial 20");
		
		eCompStyle.getStyleItems().add(eFontConstant1);
		eCompStyle.getStyleItems().add(eFontConstant2);

		Style eStyle1 = StylesFactory.eINSTANCE.createStyle();
		StyleSelector eStyleSelector1 = StylesFactory.eINSTANCE.createStyleSelector();
		eStyleSelector1.setProperty("font");
		eStyleSelector1.setSelector("normal-font");
		eStyle1.getStyleSelectors().add(eStyleSelector1);

		Style eStyle2 = StylesFactory.eINSTANCE.createStyle();
		StyleSelector eStyleSelector2 = StylesFactory.eINSTANCE.createStyleSelector();
		eStyleSelector2.setProperty("font");
		eStyleSelector2.setSelector("italic-bold-arial-large-font");
		eStyle2.getStyleSelectors().add(eStyleSelector2);

		Composite eComp = (Composite)WidgetsFactory.eINSTANCE.create(widgetsPackage.getComposite());
		eComp.eSet(stylesPackage.getStyled_Style(), eCompStyle);
		Label eLabel1 = (Label)WidgetsFactory.eINSTANCE.create(widgetsPackage.getLabel());
		eLabel1.setStyle(eStyle1);
		eLabel1.setText("label 1");
		Label eLabel2 = (Label)WidgetsFactory.eINSTANCE.create(widgetsPackage.getLabel());
		eLabel2.setStyle(eStyle2);
		eLabel2.setText("label 2");
		
		eComp.getControls().add(eLabel1);
		eComp.getControls().add(eLabel2);
		builder.build(eComp, getTopLevel());

		org.eclipse.swt.widgets.Composite comp = getChild(0, org.eclipse.swt.widgets.Composite.class);
		org.eclipse.swt.widgets.Control label1 = getChild(comp, 0, org.eclipse.swt.widgets.Control.class, false);
		org.eclipse.swt.widgets.Control label2 = getChild(comp, 1, org.eclipse.swt.widgets.Control.class, false);
		Assert.assertNotNull(label1);
		Assert.assertNotNull(label2);
		testFont(label1.getFont(), "Times", SWT.NORMAL, 12);
		testFont(label2.getFont(), "Arial", SWT.ITALIC | SWT.BOLD, 20);
	}

	public static void main(String[] args) {
		TmTestCase.doRun(LabelFontTest.class);
	}
}
