/*******************************************************************************
 * Copyright (c) 2008 Hallvard Traetteberg.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Hallvard Traetteberg - initial API and implementation
 ******************************************************************************/
package org.eclipse.e4.tm.builder.styles;

import junit.framework.Assert;

import org.eclipse.e4.tm.builder.TmModelTest;
import org.eclipse.e4.tm.builder.TmTestCase;
import org.eclipse.e4.tm.styles.ImageItem;
import org.eclipse.e4.tm.styles.Style;
import org.eclipse.e4.tm.styles.StyleSelector;
import org.eclipse.e4.tm.styles.StylesFactory;
import org.eclipse.e4.tm.styles.StylesPackage;
import org.eclipse.e4.tm.util.UtilPackage;
import org.eclipse.e4.tm.widgets.WidgetsFactory;
import org.eclipse.e4.tm.widgets.WidgetsPackage;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.swt.widgets.Label;

public class LabelImageTest extends TmModelTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	public void testLabel() {
		WidgetsPackage widgetsPackage = WidgetsPackage.eINSTANCE;
		StylesPackage stylesPackage = StylesPackage.eINSTANCE;
		Style eStyle = StylesFactory.eINSTANCE.createStyle();
		eStyle.setName("label");

		ImageItem eImageItem = StylesFactory.eINSTANCE.createImageItem();
		eImageItem.setName("my-face");
		eImageItem.setSource("platform:/plugin/org.eclipse.e4.tm.builder.tests/resources/Label.gif");
		
		eStyle.getStyleItems().add(eImageItem);

		StyleSelector eStyleSelector = StylesFactory.eINSTANCE.createStyleSelector();
		eStyleSelector.setProperty("image");
		eStyleSelector.setSelector("my-face");

		eStyle.getStyleSelectors().add(eStyleSelector);

		EObject eLabel = WidgetsFactory.eINSTANCE.create(widgetsPackage.getLabel());
		eLabel.eSet(stylesPackage.getStyled_Style(), eStyle);
		eLabel.eSet(UtilPackage.eINSTANCE.getLabeled_Text(), "label");

		builder.build(eLabel, getTopLevel());

		Label label = getChild(0, Label.class);
		Assert.assertNotNull(label);
		Assert.assertNotNull(label.getImage());
	}

	public static void main(String[] args) {
		TmTestCase.doRun(LabelImageTest.class);
	}
}
