/*******************************************************************************
 * Copyright (c) 2008 Hallvard Traetteberg.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Hallvard Traetteberg - initial API and implementation
 ******************************************************************************/
package org.eclipse.e4.tm.builder.widgets;

import java.util.List;

import junit.framework.Assert;

import org.eclipse.e4.tm.builder.TmModelTest;
import org.eclipse.e4.tm.util.UtilPackage;
import org.eclipse.e4.tm.widgets.WidgetsFactory;
import org.eclipse.e4.tm.widgets.WidgetsPackage;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

public abstract class AbstractCompositeTest extends TmModelTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected EObject[] fillWithELabels(EObject eComp, String labelText, int n) {
		EObject[] eLabels = new EObject[n];
		for (int i = 0; i < eLabels.length; i++) {
			EObject eLabel = WidgetsFactory.eINSTANCE.create(WidgetsPackage.eINSTANCE.getLabel());
			eLabel.eSet(UtilPackage.eINSTANCE.getLabeled_Text(), labelText + i);
			eLabels[i] = eLabel;
			((List<EObject>)eComp.eGet(WidgetsPackage.eINSTANCE.getAbstractComposite_Controls())).add(eLabel);
		}
		return eLabels;
	}

	protected void testLabels(Composite comp, EObject[] eLabels, String labelText, boolean ignoreNonLabels) {
		Control[] labels = comp.getChildren();
		Assert.assertEquals(eLabels.length, labels.length);
		for (int i = 0; i < eLabels.length; i++) {
			Assert.assertEquals(labelText + i, getFeature(eLabels[i], "text"));
			if (labels[i] instanceof Label) {
				Assert.assertEquals(labelText + i, ((Label)labels[i]).getText());
			} else if (labels[i] instanceof CLabel) {
					Assert.assertEquals(labelText + i, ((CLabel)labels[i]).getText());
			} else if (! ignoreNonLabels) {
				Assert.fail();
			}
		}
	}
}
