/*******************************************************************************
 * Copyright (c) 2008 Hallvard Traetteberg.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Hallvard Traetteberg - initial API and implementation
 ******************************************************************************/
package org.eclipse.e4.tm.builder.widgets;

import junit.framework.Assert;

import org.eclipse.e4.tm.builder.EventHelper;
import org.eclipse.e4.tm.builder.TmModelTest;
import org.eclipse.e4.tm.builder.TmTestCase;
import org.eclipse.e4.tm.widgets.WidgetsFactory;
import org.eclipse.e4.tm.widgets.WidgetsPackage;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;

public class ComboBoxTest extends TmModelTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	public void testCombo() {
		EObject eCombo = WidgetsFactory.eINSTANCE.create(WidgetsPackage.eINSTANCE.getComboBox());
		String comboText = "A label";
		eCombo.eSet(WidgetsPackage.eINSTANCE.getText_Text(), comboText);
		String[] comboItemStrings = {"Item 1", "Item 2", "Item 3"};
		java.util.List listItems = getListFeature(eCombo, "items");
		for (int i = 0; i < comboItemStrings.length; i++) {
			listItems.add(comboItemStrings[i]);
		}

		builder.build(eCombo, getTopLevel());

		CCombo combo = getChild(0, CCombo.class);
		Assert.assertEquals(comboText, combo.getText());
		Assert.assertEquals(3, combo.getItemCount());
		for (int i = 0; i < comboItemStrings.length; i++) {
			Assert.assertEquals(comboItemStrings[i], combo.getItem(i));
		}
		Assert.assertEquals(setFeature(eCombo, "text", "x" + comboText), combo.getText());

		listItems.clear();
		Assert.assertEquals(0, combo.getItemCount());
		for (int i = 0; i < comboItemStrings.length; i++) {
			listItems.add("x" + comboItemStrings[i]);
		}
		Assert.assertEquals(3, combo.getItemCount());
		for (int i = 0; i < comboItemStrings.length; i++) {
			Assert.assertEquals("x" + comboItemStrings[i], combo.getItem(i));
		}

		combo.select(1);
		EventHelper.sendEvent(combo, SWT.Selection);
		Assert.assertNotNull(getFeature(eCombo, "selectionEvent"));
		Assert.assertEquals(1, getFeature(eCombo, "selectionIndex"));
	}

	public static void main(String[] args) {
		TmTestCase.doRun(ComboBoxTest.class);
	}
}
