/*******************************************************************************
 * Copyright (c) 2008 Hallvard Traetteberg.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Hallvard Traetteberg - initial API and implementation
 ******************************************************************************/
package org.eclipse.e4.tm.builder.widgets;

import junit.framework.Assert;
import org.eclipse.e4.tm.builder.EventHelper;
import org.eclipse.e4.tm.builder.TmModelTest;
import org.eclipse.e4.tm.builder.TmTestCase;
import org.eclipse.e4.tm.widgets.WidgetsFactory;
import org.eclipse.e4.tm.widgets.WidgetsPackage;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.List;

public class SingleSelectionListTest extends TmModelTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	public void testList() {
		EObject eList = WidgetsFactory.eINSTANCE.create(WidgetsPackage.eINSTANCE.getSingleSelectionList());
		String[] listItemStrings = {"Item 1", "Item 2", "Item 3"};
		java.util.List listItems = getListFeature(eList, "items");
		for (int i = 0; i < listItemStrings.length; i++) {
			listItems.add(listItemStrings[i]);
		}

		builder.build(eList, getTopLevel());

		List list = getChild(0, List.class);
		testStyle(list, SWT.SINGLE);
		Assert.assertEquals(3, list.getItemCount());
		for (int i = 0; i < listItemStrings.length; i++) {
			Assert.assertEquals(listItemStrings[i], list.getItem(i));
		}

		listItems.clear();
		Assert.assertEquals(0, list.getItemCount());
		for (int i = 0; i < listItemStrings.length; i++) {
			listItems.add("x" + listItemStrings[i]);
		}
		Assert.assertEquals(3, list.getItemCount());
		for (int i = 0; i < listItemStrings.length; i++) {
			Assert.assertEquals("x" + listItemStrings[i], list.getItem(i));
		}

		list.setSelection(1);
		EventHelper.sendEvent(list, SWT.Selection);
		Assert.assertNotNull(getFeature(eList, "selectionEvent"));
		Assert.assertEquals(1, getFeature(eList, "selectionIndex"));
	}

	public static void main(String[] args) {
		TmTestCase.doRun(SingleSelectionListTest.class);
	}
}
