/*******************************************************************************
 * Copyright (c) 2008 Hallvard Traetteberg.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Hallvard Traetteberg - initial API and implementation
 ******************************************************************************/
package org.eclipse.e4.tm.builder.widgets;

import java.util.List;

import junit.framework.Assert;

import org.eclipse.e4.tm.builder.TmTestCase;
import org.eclipse.e4.tm.widgets.WidgetsFactory;
import org.eclipse.e4.tm.widgets.WidgetsPackage;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

public class SplitPaneTest extends AbstractCompositeTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	public void testHorizontalSplitPane() {
		testSplitPane("horizontal", SWT.HORIZONTAL);
	}
	public void testVerticalSplitPane() {
		testSplitPane("vertical", SWT.VERTICAL);
	}
	
	private void testSplitPane(String orientationName, int orientation) {
		WidgetsPackage widgetsPackage = WidgetsPackage.eINSTANCE;
		EObject eSplitPane = WidgetsFactory.eINSTANCE.create(widgetsPackage.getSplitPane());
		eSplitPane.eSet(widgetsPackage.getSplitPane_Orientation(), orientationName);
		
		EObject eComp1 = WidgetsFactory.eINSTANCE.create(widgetsPackage.getComposite());
		EObject eComp2 = WidgetsFactory.eINSTANCE.create(widgetsPackage.getComposite());
		((List)eSplitPane.eGet(widgetsPackage.getAbstractComposite_Controls())).add(eComp1);
		((List)eSplitPane.eGet(widgetsPackage.getAbstractComposite_Controls())).add(eComp2);
		
		builder.build(eSplitPane, getTopLevel());

		SashForm sashForm = getChild(0, SashForm.class);
		Assert.assertNotNull(sashForm);
		int actualOrientation = sashForm.getOrientation();
		assertEquals(orientation, actualOrientation);

		Control[] children = sashForm.getChildren(); 
		assertEquals(2, children.length);
		assertTrue(children[0] instanceof Composite);
		assertTrue(children[1] instanceof Composite);
	}

	public static void main(String[] args) {
		TmTestCase.doRun(SplitPaneTest.class);
	}
}
