/*******************************************************************************
 * Copyright (c) 2009 Hallvard Traetteberg.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Hallvard Traetteberg - initial API and implementation
 ******************************************************************************/
package org.eclipse.e4.tm.graphics.tests;
/*******************************************************************************
 * Copyright (c) 2008 Hallvard Traetteberg.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Hallvard Traetteberg - initial API and implementation
 ******************************************************************************/

import junit.framework.Assert;

import org.eclipse.e4.tm.builder.TmModelTest;
import org.eclipse.e4.tm.builder.TmTestCase;
import org.eclipse.e4.tm.graphics.util.Rectangle;
import org.eclipse.e4.tm.graphics2d.Canvas;
import org.eclipse.e4.tm.graphics2d.Graphics2dFactory;
import org.eclipse.e4.tm.graphics2d.Graphics2dPackage;
import org.eclipse.e4.tm.graphics2d.Layer2d;
import org.eclipse.e4.tm.graphics2d.Rect2d;
import org.eclipse.e4.tm.graphics2d.Text2d;
import org.eclipse.swt.widgets.Composite;

import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PRect;
import edu.umd.cs.piccolo.nodes.PText;

public class Graphical2dTest extends TmModelTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected <T> T getChild(PNode node, int i, Class<T> c) {
		PNode child = node.getChild(i);
		return (c.isInstance(child) ? (T)child : null);
	}

	public void testPNodes() {
		Graphics2dPackage graphicsPackage = Graphics2dPackage.eINSTANCE;
		Canvas parent = (Canvas)Graphics2dFactory.eINSTANCE.create(graphicsPackage.getCanvas());
		Layer2d eLayer = (Layer2d)Graphics2dFactory.eINSTANCE.create(graphicsPackage.getLayer2d());
		eLayer.setBounds(new Rectangle());
		parent.getLayers().add(eLayer);
		
		Text2d eTextNode = (Text2d)Graphics2dFactory.eINSTANCE.create(graphicsPackage.getText2d());
		eTextNode.setBounds(new Rectangle());
		eTextNode.setText("A text node");
		eLayer.getChildren().add(eTextNode);

		Rect2d eRectNode = (Rect2d)Graphics2dFactory.eINSTANCE.create(graphicsPackage.getRect2d());
		eRectNode.setBounds(new Rectangle(10, 10, 30, 30));
		eLayer.getChildren().add(eRectNode);

		builder.build(parent, getTopLevel());

		Composite comp = getChild(0, Composite.class);
		Assert.assertNotNull(comp);
		
		assertEquals(comp, builder.getObject(parent, Composite.class));
		PCanvas pCanvas = builder.getObject(parent, PCanvas.class);

		assertEquals(pCanvas.getCamera().getLayerCount(), 1);
		PLayer layer = pCanvas.getCamera().getLayer(0);
		assertEquals(pCanvas.getLayer(), layer);
	
		PText pTextNode = getChild(layer, 0, PText.class);
		Assert.assertNotNull(pTextNode);
		assertEquals(eTextNode.getText(), pTextNode.getText());
		
		PRect pRectNode = getChild(layer, 1, PRect.class);
		Assert.assertNotNull(pRectNode);
		testRectangle(eRectNode.getBounds(), pRectNode.getBoundsReference());
	}

	private void testRectangle(Rectangle expected, Rectangle actual) {
		assertEquals(expected.x, actual.x);
		assertEquals(expected.y, actual.y);
		assertEquals(expected.width, actual.width);
		assertEquals(expected.height, actual.height);
	}

	public static void main(String[] args) {
		TmTestCase.doRun(Graphical2dTest.class);
	}
}
