/*******************************************************************************
 * Copyright (c) 2010 SAP AG, Walldorf
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    SAP AG - initial API and implementation
 *******************************************************************************/
package org.eclipse.platform.discovery.compatibility.internal.contributors.impl;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.core.runtime.IContributor;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.platform.discovery.compatibility.internal.contributors.ContributionFailedException;
import org.eclipse.platform.discovery.compatibility.internal.contributors.IDynamicRegistryContributor;
import org.eclipse.platform.discovery.compatibility.internal.readers.ISearchPageDescription;
import org.eclipse.platform.discovery.compatibility.internal.readers.ISearchPageParser;
import org.eclipse.platform.discovery.runtime.internal.xp.IObjectTypeExtensionParser;
import org.w3c.dom.Element;

public class DynamicObjectTypeContributor extends DynamicRegistryContributor implements IDynamicRegistryContributor {
	
	private final ISearchPageParser searchPageParser;

	
	public DynamicObjectTypeContributor(ISearchPageParser searchPageParser) {
		this.searchPageParser = searchPageParser;
	}


	@Override
	public void contribute(IExtensionRegistry registry, IContributor contributor, Object token) throws ContributionFailedException {
		
		Collection<ISearchPageDescription> failedContributors = new ArrayList<ISearchPageDescription>();
		for (ISearchPageDescription pageDescription : searchPageParser.readContributions()) {

			Element plugin = xmlUtils().createPluginSnippet(IObjectTypeExtensionParser.XP_ID);

			addObjectTypeElement(pageDescription, plugin);
			
			String name = MessageFormat.format("Dynamic ObjectType for search page: {0}", pageDescription.toString()); //$NON-NLS-1$
			
			boolean success = writeToRegistry(plugin, registry, contributor, token, name);
			
			if(!success) {
				/*
				 * it's very likely the implementation will never reach here. the underlying registry returns false if
				 * there was a parsing problem with the contribution (unexpected because the XML was generated by us)
				 * or an IOException (also unexpected, we're only dealing with in-memory objects).
				 * In case flow does reach here, create an error message and continue with the loop, contributing the rest of the stuff on a
				 * best-effort basis.
				 */
				failedContributors.add(pageDescription);
			}
		}
		
		if(!failedContributors.isEmpty()) {
			throw new ContributionFailedException("Dynamic object type contribution failed", failedContributors); //$NON-NLS-1$
		}

	}


	/* adds an object type element to the plugin snippet passed, using the provided pageDescription*/
	private void addObjectTypeElement(ISearchPageDescription pageDescription, Element plugin) {
		Element objecttype = plugin.getOwnerDocument().createElement(IObjectTypeExtensionParser.XP_ELEMENT_NAME);
		objecttype.setAttribute(IObjectTypeExtensionParser.ID_ATTR_NAME, pageDescription.getId());
		objecttype.setAttribute(IObjectTypeExtensionParser.DISPLAY_NAME_ATTR_NAME, pageDescription.getDisplayName());

		Element extension = (Element)plugin.getFirstChild();
		extension.appendChild(objecttype);
	}


}