/*******************************************************************************
 * Copyright (c) 2010 SAP AG, Walldorf
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    SAP AG - initial API and implementation
 *******************************************************************************/
package org.eclipse.platform.discovery.compatibility.internal.contributors.impl;

import java.util.ResourceBundle;


import org.eclipse.core.runtime.IContributor;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.platform.discovery.compatibility.internal.DiscoveryCompatibilityPlugin;
import org.eclipse.platform.discovery.util.internal.StringInputStreamAdapter;
import org.eclipse.platform.discovery.util.internal.logging.ILogger;
import org.eclipse.platform.discovery.util.internal.xml.IPluginXmlUtils;
import org.eclipse.platform.discovery.util.internal.xml.PluginXmlUtils;
import org.w3c.dom.Element;

public abstract class DynamicRegistryContributor {
	
	private static final boolean PERSIST = false;

	
	/**
	 * Contributes the passed plugin.xml snippet to the extension registry.
	 * @param plugin - the plugin.xml to contribute. This method expects that any string literals eligible for translation are already translated to the current locale.
	 * This method will make no additional translation efforts, meaning registry.addContribution will be invoked with a null resource bundle.
	 * @return
	 */
	protected boolean writeToRegistry(Element plugin, IExtensionRegistry registry, IContributor contributor, Object token,  String name) {
		
		String xml = xmlUtils().toXML(plugin);
		StringInputStreamAdapter inputStream = new StringInputStreamAdapter(xml);

		/* generated plugin is expected to be already translated, so we do not need a translation bundle */
		ResourceBundle translationBundle = null;
		return registry.addContribution(inputStream, contributor, PERSIST, name, translationBundle, token);

	}
	
	protected ILogger logger() {
		return DiscoveryCompatibilityPlugin.getDefault().logger();
	}

	protected IPluginXmlUtils xmlUtils() {
		return new PluginXmlUtils();
	}
}
