/*******************************************************************************
 * Copyright (c) 2010 SAP AG, Walldorf
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    SAP AG - initial API and implementation
 *******************************************************************************/
package org.eclipse.platform.discovery.compatibility.internal.readers.impl;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.platform.discovery.compatibility.internal.readers.ISearchPageDescription;
import org.eclipse.platform.discovery.compatibility.internal.readers.ISearchPageParser;
import org.eclipse.platform.discovery.runtime.internal.xp.impl.AbstractExtensionPointParser;

public class SearchPageParser extends AbstractExtensionPointParser implements ISearchPageParser{

	static final String ID = "id"; //$NON-NLS-1$
	static final String LABEL = "label"; //$NON-NLS-1$
	
	private static final String PAGE = "page"; //$NON-NLS-1$
	private static final String SEARCH_PAGES_XP_ID = "org.eclipse.search.searchPages"; //$NON-NLS-1$

	public SearchPageParser(IExtensionRegistry extensionRegistry) {
		super(extensionRegistry, SEARCH_PAGES_XP_ID, PAGE);
	}
	
	@Override
	public Set<ISearchPageDescription> readContributions() {
		Set<ISearchPageDescription> result = new HashSet<ISearchPageDescription>();
		for(IConfigurationElement element: getConfigurationElements()) {
			//note that according to the javadoc of IConfigurationElement, the obtained attributes will be automatically translated
			String label = element.getAttribute(LABEL);
			String id = element.getAttribute(ID);
			result.add(new SearchPageDescription(id, label));
		}
			
		return result;
	}
	
	
	
	
}
