/*******************************************************************************
 * Copyright (c) 2010 SAP AG, Walldorf.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SAP AG - initial API and implementation
 *******************************************************************************/
package org.eclipse.platform.discovery.integration.internal;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.platform.discovery.runtime.internal.impl.XpParsersFactory;
import org.eclipse.platform.discovery.runtime.internal.xp.IContributionsReader;
import org.eclipse.platform.discovery.runtime.internal.xp.IGenericXpParser;
import org.eclipse.platform.discovery.util.internal.logging.ILogger;
import org.eclipse.platform.discovery.util.internal.logging.Logger;


/**
 * Generic parser for the
 * 
 * @author Danail Branekov
 * 
 * @param <T>
 */
public abstract class ViewCustXpParser<T> implements IContributionsReader<T>
{
	private final XpParsersFactory xpParsersFactory;
	private final IExtensionRegistry extensionRegistry;

	public ViewCustXpParser(final IExtensionRegistry extRegistry)
	{
		xpParsersFactory = new XpParsersFactory();
		this.extensionRegistry = extRegistry;
	}

	protected IGenericXpParser genericXpParser()
	{
		return xpParsersFactory.createGenericXpParser();
	}

	protected ILogger logger()
	{
		return Logger.instance();
	}

	@SuppressWarnings("unchecked")
	@Override
	public Set<T> readContributions()
	{
		final Set<T> result = new HashSet<T>();
		final List<IConfigurationElement> configElements = genericXpParser().getConfigurationElements(extensionRegistry, xpId(), elementName());
		for (IConfigurationElement configElem : configElements)
		{
			try
			{
				result.add((T) configElem.createExecutableExtension(implementationAttributeName()));
			} catch (CoreException e)
			{
				logger().logError(e.getMessage(), e);
			}
		}

		return result;
	}

	/**
	 * Extenders should implement this method to specify the attribute name which stands for the instantiatable implementation
	 */
	protected abstract String implementationAttributeName();
	
	/**
	 * Extenders should implement this method to specify the name of the element they are interested in 
	 * @return
	 */
	protected abstract String elementName();
	
	/**
	 * Extenders should implement this method to return the ID of the extension point this parser will process
	 * @return extension point id
	 */
	protected abstract String xpId();
}
