/*******************************************************************************
 * Copyright (c) 2010 SAP AG, Walldorf.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SAP AG - initial API and implementation
 *******************************************************************************/
package org.eclipse.platform.discovery.runtime.internal;

import org.eclipse.platform.discovery.runtime.internal.model.descriptions.IDestinationCategoryDescription;
import org.eclipse.platform.discovery.runtime.internal.model.descriptions.IObjectTypeDescription;
import org.eclipse.platform.discovery.runtime.internal.model.descriptions.ISearchProviderDescription;
import org.eclipse.platform.discovery.runtime.internal.xp.IDestinationCategoryExtensionParser;
import org.eclipse.platform.discovery.runtime.internal.xp.IDestinationsProviderExtensionParser;
import org.eclipse.platform.discovery.runtime.internal.xp.IObjectTypeExtensionParser;
import org.eclipse.platform.discovery.runtime.internal.xp.ISearchProvidersExtensionParser;

/**
 * Interface for reading and configuring the active search provider for the couple object type - search destination<br>
 * For a given couple object type - search destination there might be at most one search provider which will be used when searching for the given
 * object type in the given destination
 * 
 * @author Danail Branekov
 */
public interface ISearchProviderActivationConfig
{
	/**
	 * Get the search provider ID which is active for the object type and destination category specified
	 * 
	 * @param objectType
	 *            the object type
	 * @param destinationCategory
	 *            the destination category
	 * @return the description of the provider which is active for the specified object type and destination or <code>null</code> if none
	 */
	public ISearchProviderDescription getActiveSearchProviderDescription(final IObjectTypeDescription objectType,
									final IDestinationCategoryDescription destinationCategory) throws ProviderNotFoundException;

	/**
	 * Retrieve a search provider extension parser instance
	 * 
	 * @see ISearchProvidersExtensionParser
	 */
	public ISearchProvidersExtensionParser getSearchProvidersExtensionParser();

	/**
	 * Retrieve an object type extension parser instance
	 * 
	 * @see IObjectTypeExtensionParser
	 */
	public IObjectTypeExtensionParser getObjectTypeExtensionParser();

	/**
	 * Retrieve a destination categories extension parser instance
	 * 
	 * @return
	 */
	public IDestinationCategoryExtensionParser getDestinationCategoryExtensionParser();
	
	/**
	 * Retrieve a destination categories extension parser instance
	 */
	public IDestinationsProviderExtensionParser getDestinationsProviderExtensionParser();
}
