/*******************************************************************************
 * Copyright (c) 2010 SAP AG, Walldorf.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SAP AG - initial API and implementation
 *******************************************************************************/
package org.eclipse.platform.discovery.runtime.internal.xp;

import java.util.List;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;

/**
 * Interface for a generic extension point parser
 * 
 * @author Danail Branekov
 */
public interface IGenericXpParser
{
	/**
	 * Obtains a list of {@link IConfigurationElement} instances for the extension point and element name specified<br>
	 * In order this method to work correctly, contribution to the extension point should look like:
	 * 
	 * <pre>
	 * &lt;extension
	 *          point=&quot;extension_point_id&quot;&gt;
	 *       &lt;element_name
	 *             attribute_name=&quot;attribute_value&quot;
	 *       &lt;/element_name&gt;
	 *    &lt;/extension&gt;
	 * </pre>
	 * 
	 * @param extRegistry the extension registry
	 * @param xpId the extension point id
	 * @param elementName the element name
	 * @return
	 */
	public List<IConfigurationElement> getConfigurationElements(final IExtensionRegistry extRegistry, final String xpId, final String elementName);
}
