/*******************************************************************************
 * Copyright (c) 2010 SAP AG, Walldorf.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SAP AG - initial API and implementation
 *******************************************************************************/
package org.eclipse.platform.discovery.runtime.internal.xp.impl;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.platform.discovery.runtime.internal.model.descriptions.IObjectTypeDescription;
import org.eclipse.platform.discovery.runtime.internal.model.descriptions.impl.ObjectTypeDescription;
import org.eclipse.platform.discovery.runtime.internal.xp.IObjectTypeExtensionParser;


public class ObjectTypeExtensionParser extends AbstractExtensionPointParser implements IObjectTypeExtensionParser
{

	public ObjectTypeExtensionParser()
	{
		super(Platform.getExtensionRegistry(), XP_ID, XP_ELEMENT_NAME);
	}
	
	public ObjectTypeExtensionParser(IExtensionRegistry registry) {
		super(registry, XP_ID, XP_ELEMENT_NAME);
	}

	public Set<IObjectTypeDescription> readContributions()
	{
		final Set<IObjectTypeDescription> result = new HashSet<IObjectTypeDescription>();
		for(IConfigurationElement element : getConfigurationElements())
		{
			result.add(createObjectType(element));
		}
		
		return result;
	}

	private IObjectTypeDescription createObjectType(final IConfigurationElement element)
	{
		return new ObjectTypeDescription(element.getAttribute(ID_ATTR_NAME), element.getAttribute(DISPLAY_NAME_ATTR_NAME));
	}
}
