/*******************************************************************************
 * Copyright (c) 2010 SAP AG, Walldorf.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SAP AG - initial API and implementation
 *******************************************************************************/
package org.eclipse.platform.discovery.ui.internal.xp.impl;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.platform.discovery.runtime.internal.impl.XpParsersFactory;
import org.eclipse.platform.discovery.runtime.internal.xp.IGenericXpParser;
import org.eclipse.platform.discovery.ui.api.IAdvancedSearchParamsUiContributor;
import org.eclipse.platform.discovery.ui.internal.search.advancedparams.IAdvancedSearchParamsUiContributorDescr;
import org.eclipse.platform.discovery.ui.internal.xp.IAdvancedSearchParamsUiContribXpParser;


/**
 * Implementation of the {@link IAdvancedSearchParamsUiContribXpParser} interface
 * 
 * @author Danail Branekov
 */
public class AdvancedSearchParamsUiContribXpParser implements IAdvancedSearchParamsUiContribXpParser
{
	final private IExtensionRegistry extensionRegistry;

	private static final String XP_ID = "org.eclipse.platform.discovery.integration.advancedsearchparams"; //$NON-NLS-1$
	private static final String XP_ELEMENT_NAME = "advancedparams"; //$NON-NLS-1$
	private static final String ID_ATTR_NAME = "id"; //$NON-NLS-1$
	private static final String SEARCH_PROVIDER_ID_ATTR_NAME = "searchproviderid"; //$NON-NLS-1$
	private static final String CONTRIBUTOR_CLASS_NAME_ATTR_NAME = "uicontributorclass"; //$NON-NLS-1$

	public AdvancedSearchParamsUiContribXpParser(final IExtensionRegistry extRegistry)
	{
		this.extensionRegistry = extRegistry;
	}

	public Set<IAdvancedSearchParamsUiContributorDescr> readContributions()
	{
		final Set<IAdvancedSearchParamsUiContributorDescr> result = new HashSet<IAdvancedSearchParamsUiContributorDescr>();
		for (final IConfigurationElement configElement : configurationElements())
		{
			result.add(new IAdvancedSearchParamsUiContributorDescr()
			{

				public IAdvancedSearchParamsUiContributor createContributor()
				{
					try
					{
						return (IAdvancedSearchParamsUiContributor) configElement.createExecutableExtension(CONTRIBUTOR_CLASS_NAME_ATTR_NAME);
					} catch (CoreException e)
					{
						throw new IllegalStateException(e);
					}
				}

				public String getId()
				{
					return configElement.getAttribute(ID_ATTR_NAME);
				}

				public String getSearchProviderId()
				{
					return configElement.getAttribute(SEARCH_PROVIDER_ID_ATTR_NAME);
				}
			});
		}

		return result;
	}

	private List<IConfigurationElement> configurationElements()
	{
		return genericXpParser().getConfigurationElements(this.extensionRegistry, XP_ID, XP_ELEMENT_NAME);
	}

	protected IGenericXpParser genericXpParser()
	{
		return (new XpParsersFactory()).createGenericXpParser();
	}
}
